from django.contrib.auth import get_user_model
from django.contrib.auth.models import Permission
from django.contrib.auth.backends import ModelBackend
from django.db.models import Exists, OuterRef, Q
from djoser.conf import settings

User = get_user_model()


class LoginFieldBackend(ModelBackend):

    def authenticate(self, request, username=None, password=None, **kwargs):
        if username is None:
            username = kwargs.get(settings.LOGIN_FIELD)
        if username is None or password is None:
            return
        user = User.objects.filter(**kwargs).first()
        if user and not user.check_password(password):
            return
        if user and user.is_active:
            return user


from django.contrib.auth.tokens import default_token_generator
from templated_mail.mail import BaseEmailMessage

from djoser import utils
from djoser.conf import settings


class BaseDjoserEmail(BaseEmailMessage):
    def get_context_data(self):
        context = super().get_context_data()
        overridable = {
            "protocol": settings.EMAIL_FRONTEND_PROTOCOL,
            "domain": settings.EMAIL_FRONTEND_DOMAIN,
            "site_name": settings.EMAIL_FRONTEND_SITE_NAME,
        }
        for context_key, context_value in overridable.items():
            if context_value:
                context.update({context_key: context_value})
        return context


class ActivationEmail(BaseDjoserEmail):
    template_name = "email/activation.html"

    def get_context_data(self):
        # ActivationEmail can be deleted
        context = super().get_context_data()

        user = context.get("user")
        context["uid"] = utils.encode_uid(user.pk)
        context["token"] = default_token_generator.make_token(user)
        context["url"] = settings.ACTIVATION_URL.format(**context)
        return context


class ConfirmationEmail(BaseDjoserEmail):
    template_name = "email/confirmation.html"


class PasswordResetEmail(BaseDjoserEmail):
    template_name = "email/password_reset.html"

    def get_context_data(self):
        # PasswordResetEmail can be deleted
        context = super().get_context_data()

        user = context.get("user")
        context["uid"] = utils.encode_uid(user.pk)
        context["token"] = default_token_generator.make_token(user)
        context["url"] = settings.PASSWORD_RESET_CONFIRM_URL.format(**context)
        return context


class PasswordChangedConfirmationEmail(BaseDjoserEmail):
    template_name = "email/password_changed_confirmation.html"


class UsernameChangedConfirmationEmail(BaseDjoserEmail):
    template_name = "email/username_changed_confirmation.html"


class UsernameResetEmail(BaseDjoserEmail):
    template_name = "email/username_reset.html"

    def get_context_data(self):
        context = super().get_context_data()

        user = context.get("user")
        context["uid"] = utils.encode_uid(user.pk)
        context["token"] = default_token_generator.make_token(user)
        context["url"] = settings.USERNAME_RESET_CONFIRM_URL.format(**context)
        return context



from django.urls import re_path

from . import views

urlpatterns = [
    re_path(
        r"^signup_request/$",
        views.SingupRequestView.as_view(),
        name="webauthn_signup_request",
    ),
    re_path(
        r"^signup/(?P<ukey>.+)/$", views.SignupView.as_view(), name="webauthn_signup"
    ),
    re_path(
        r"^login_request/$",
        views.LoginRequestView.as_view(),
        name="webauthn_login_request",
    ),
    re_path(r"^login/$", views.LoginView.as_view(), name="webauthn_login"),
]



from django.conf import settings
from django.db import models


class CredentialOptions(models.Model):
    challenge = models.TextField()
    username = models.TextField(unique=True)
    display_name = models.TextField()
    ukey = models.TextField(unique=True)
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        related_name="credential_options",
        null=True,
        on_delete=models.CASCADE,
    )
    credential_id = models.TextField()
    sign_count = models.IntegerField(null=True)
    public_key = models.TextField()
